<?php
/**
 * 检查授权
 */
function initialize(array $params)
{
	$domain = $_SERVER['HTTP_HOST'];
	$beginChars = substr($domain, 0, 4);
	$endChars = substr($domain, 0 - 4, 4);
	$reversal = strrev($domain);
	$md5 = md5(md5(md5($domain . $reversal) . md5($beginChars) . md5($endChars)));

	if (trim($params['configoption4']) !== $md5) {
		exit('Invalid License.');
	}

	$query['RECYCLE'] = 'SELECT `port` FROM `recycle_bin` ORDER BY `created_at` DESC LIMIT 1';
	$query['DELETE_RECYCLE'] = 'DELETE FROM `recycle_bin` WHERE `port` = :port';
	$query['ADD_RECYCLE'] = 'INSERT INTO `recycle_bin`(`port`,`created_at`) VALUES (:port,UNIX_TIMESTAMP())';
	$query['LATEST_USER'] = 'SELECT `port` FROM `user` ORDER BY `port` DESC LIMIT 1';
	$query['CREATE_ACCOUNT'] = 'INSERT INTO `user`(`passwd`,`transfer_enable`,`port`,`created_at`,`need_reset`,`sid`) VALUES (:passwd,:transfer_enable,:port,UNIX_TIMESTAMP(),:need_reset,:sid)';
	$query['ALREADY_EXISTS'] = 'SELECT `port`,`sid` FROM `user` WHERE `sid` = :sid';
	$query['ENABLE'] = 'UPDATE `user` SET `enable` = :enable WHERE `sid` = :sid';
	$query['DELETE_ACCOUNT'] = 'DELETE FROM `user` WHERE `sid` = :sid';
	$query['CHANGE_PASSWORD'] = 'UPDATE `user` SET passwd = :passwd WHERE `sid` = :sid';
	$query['USERINFO'] = 'SELECT `id`,`passwd`,`port`,`t`,`u`,`d`,`transfer_enable`,`enable`,`created_at`,`updated_at`,`need_reset`,`sid` FROM `user` WHERE `sid` = :sid';
	$query['RESET'] = 'UPDATE `user` SET `u`=0,`d`=0 WHERE `sid` = :sid';
	$query['CHANGE_PACKAGE'] = 'UPDATE `user` SET `transfer_enable` = :transfer_enable WHERE `sid` = :sid';
	return $query;
}

/**
 * 单位转换
 * @param  int $number 原始流量
 * @param  string $from   原始单位
 * @param  string $to     转换为代码
 * @return int 转换后的值
 */
function convert($number, $from, $to)
{
	$to = strtolower($to);
	$from = strtolower($from);

	switch ($from) {
	case 'gb':
		switch ($to) {
		case 'mb':
			return $number * 1024;
		case 'bytes':
			return $number * 1073741824;
		default:
			return $number;
		}

		break;

	case 'mb':
		switch ($to) {
		case 'gb':
			return $number / 1024;
		case 'bytes':
			return $number * 1048576;
		default:
			return $number;
		}

		break;

	case 'bytes':
		switch ($to) {
		case 'gb':
			return $number / 1073741824;
		case 'mb':
			return $number / 1048576;
		default:
			return $number;
		}

		break;

	default:
		return $number;
	}
}
/**
 * 定义module 相关描述
 * @return array
 */
function shadowsocks_MetaData()
{
	return array('DisplayName' => 'Shadowsocks for whmcs', 'RequiresServer' => true);
}

/**
 * 设置插件的配置选项
 * @return array
 */
function shadowsocks_ConfigOptions()
{
	return array(
	'数据库名' => array('Type' => 'text', 'Size' => '25'),
	'重置流量' => array(
		'Type'        => 'dropdown',
		'Options'     => array('1' => '需要重置', '0' => '不需要重置'),
		'Description' => '是否需要重置流量'
		),
	'流量限制' => array('Type' => 'text', 'Size' => '25', 'Description' => '单位MB'),
	'授权密钥' => array('Type' => 'text', 'Size' => '32', 'Description' => '请输入您从购买者手中获取的密钥'),
	'起始端口' => array('Type' => 'text', 'Size' => '25', 'Description' => '如果数据库有记录此项无效'),
	'线路列表' => array('Type' => 'textarea', 'Rows' => '3', 'Cols' => '50', 'Description' => '格式 xxx|服务器地址|加密方式 一行一个')
	);
}

/**
 * 测试链接
 * @param  array $params common module parameters
 * @return array
 */
function shadowsocks_TestConnection(array $params)
{
	try {
		$dbhost = $params['serverip'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		new PDO('mysql:host=' . $dbhost, $dbuser, $dbpass);
		$success = true;
		$errorMsg = '';
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_TestConnection', $params, $e->getMessage(), $e->getTraceAsString());
		$success = false;
		$errorMsg = $e->getMessage();
	}

	return array('success' => $success, 'error' => $errorMsg);
}

/**
 * 创建账号
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_CreateAccount(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$already = $db->prepare($query['ALREADY_EXISTS']);
		$already->bindValue(':sid', $params['serviceid']);
		$already->execute();

		if ($already->fetchColumn()) {
			return 'User already exists.';
		}

		$bandwidth = (!empty($params['configoption3']) ? convert($params['configoption3'], 'mb', 'bytes') : (!empty($params['configoptions']['traffic']) ? convert($params['configoptions']['traffic'], 'gb', 'bytes') : '1099511627776'));
		$recycle = $db->query($query['RECYCLE']);
		$recycle = $recycle->fetch();

		if ($recycle) {
			$port = $recycle['port'];
			define('RECYCLE', true);
		}
		else {
			$port = $db->query($query['LATEST_USER']);
			$port = $port->fetch();

			if ($port) {
				$port = $port['port'] + 1;
			}
			else {
				$port = (!empty($params['configoption5']) ? $params['configoption5'] : '10000');
			}
		}

		$create = $db->prepare($query['CREATE_ACCOUNT']);
		$create->bindValue(':passwd', $params['customfields']['password']);
		$create->bindValue(':transfer_enable', $bandwidth);
		$create->bindValue(':port', $port);
		$create->bindValue(':need_reset', $params['configoption2']);
		$create->bindValue(':sid', $params['serviceid']);
		$todo = $create->execute();

		if ($todo) {
			if (defined('RECYCLE') && RECYCLE) {
				$recycle = $db->prepare($query['DELETE_RECYCLE']);
				$recycle->bindParam(':port', $port);
				$recycle->execute();
			}
		}
		else {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_CreateAccount', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 停用账号
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_SuspendAccount(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$enable = $db->prepare($query['ENABLE']);
		$enable->bindValue(':enable', '0');
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_SuspendAccount', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 恢复账号
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_UnsuspendAccount(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$enable = $db->prepare($query['ENABLE']);
		$enable->bindValue(':enable', '1');
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_UnsuspendAccount', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 删除账号
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_TerminateAccount(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$already = $db->prepare($query['ALREADY_EXISTS']);
		$already->bindValue(':sid', $params['serviceid']);
		$already->execute();
		$port = $already->fetch();

		if ($port) {
			$recycle = $db->prepare($query['ADD_RECYCLE']);
			$recycle->bindValue(':port', $port['port']);
			$todo = $recycle->execute();

			if (!$todo) {
				$error = $db->errorInfo();
				return $error[2];
			}
		}
		else {
			return 'User does not exists.';
		}

		$enable = $db->prepare($query['DELETE_ACCOUNT']);
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_TerminateAccount', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 更改套餐
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_ChangePackage(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$bandwidth = (!empty($params['configoption3']) ? convert($params['configoption3'], 'mb', 'bytes') : (!empty($params['configoptions']['traffic']) ? convert($params['configoptions']['traffic'], 'gb', 'bytes') : '1099511627776'));
		$enable = $db->prepare($query['CHANGE_PACKAGE']);
		$enable->bindValue(':transfer_enable', $bandwidth);
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_ChangePackage', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 更改密码
 * @param  array  $params common module parameters
 * @return string
 */
function shadowsocks_ChangePassword(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$enable = $db->prepare($query['CHANGE_PASSWORD']);
		$enable->bindValue(':passwd', $params['password']);
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_ChangePassword', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 添加一个自定义功能
 * @return array 添加一个自定义的限制功能
 */
function shadowsocks_AdminCustomButtonArray()
{
	return array('Reset' => 'ResetBandwidth');
}

/**
 * 手动重置流量
 * @return string
 */
function shadowsocks_ResetBandwidth(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$enable = $db->prepare($query['RESET']);
		$enable->bindValue(':sid', $params['serviceid']);
		$todo = $enable->execute();

		if (!$todo) {
			$error = $db->errorInfo();
			return $error[2];
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_ResetBandwidth', $params, $e->getMessage(), $e->getTraceAsString());
		return $e->getMessage();
	}

	return 'success';
}

/**
 * 前台界面
 * @param  array $params common module parameters
 * @return array
 */
function shadowsocks_ClientArea(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$usage = $db->prepare($query['USERINFO']);
		$usage->bindValue(':sid', $params['serviceid']);
		$usage->execute();
		$usage = $usage->fetch();
		$nodes = $params['configoption6'];
		$nodes = explode("\n", $nodes);
		$results = array();

		foreach ($nodes as $node) {
			$results[] = explode('|', $node);
		}

		$user = array('passwd' => $usage['passwd'], 'port' => $usage['port'], 'u' => $usage['u'], 'd' => $usage['d'], 't' => $usage['t'], 'sum' => $usage['u'] + $usage['d'], 'transfer_enable' => $usage['transfer_enable'], 'created_at' => $usage['created_at'], 'updated_at' => $usage['updated_at']);
		if ($usage && $usage['enable']) {
			return array(
	'tabOverviewReplacementTemplate' => 'details.tpl',
	'templateVariables'              => array('usage' => $user, 'params' => $params, 'nodes' => $results)
	);
		}

		return array(
	'tabOverviewReplacementTemplate' => 'error.tpl',
	'templateVariables'              => array('usefulErrorHelper' => '出现了一些问题，可能您的服务还未开通，请稍后再来试试。')
	);
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_ClientArea', $params, $e->getMessage(), $e->getTraceAsString());
		return array(
	'tabOverviewReplacementTemplate' => 'error.tpl',
	'templateVariables'              => array('usefulErrorHelper' => $e->getMessage())
	);
	}
}

/**
 * 后台管理界面
 * @param  array  $params common module parameters
 * @return array
 */
function shadowsocks_AdminServicesTabFields(array $params)
{
	$query = initialize($params);

	try {
		$dbhost = $params['serverip'];
		$dbname = $params['configoption1'];
		$dbuser = $params['serverusername'];
		$dbpass = $params['serverpassword'];
		$db = new PDO('mysql:host=' . $dbhost . ';dbname=' . $dbname, $dbuser, $dbpass);
		$userinfo = $db->prepare($query['USERINFO']);
		$userinfo->bindValue(':sid', $params['serviceid']);
		$userinfo->execute();
		$userinfo = $userinfo->fetch();

		if ($userinfo) {
			return array('端口' => $userinfo['port'], '流量' => convert($userinfo['transfer_enable'], 'bytes', 'mb') . 'MB', '上传' => round(convert($userinfo['u'], 'bytes', 'mb')) . 'MB', '下载' => round(convert($userinfo['d'], 'bytes', 'mb')) . 'MB', '已用' => round(convert($userinfo['d'] + $userinfo['u'], 'bytes', 'mb')) . 'MB', '最后使用' => date('Y-m-d H:i:s', $userinfo['t']), '上次重置' => date('Y-m-d H:i:s', $userinfo['updated_at']));
		}
	}
	catch (Exception $e) {
		logModuleCall('shadowsocks', 'shadowsocks_AdminServicesTabFields', $params, $e->getMessage(), $e->getTraceAsString());
	}

	return array();
}

defined('WHMCS') || exit('This file cannot be accessed directly');

?>
